/*
 * Copyright (c) 2022 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/kernel.h>
#include <zephyr/device.h>

#include <zephyr/ipc/ipc_service.h>
#include <hal/nrf_reset.h>
#include <string.h>
#include <zephyr/logging/log.h>

#include <zephyr/usb/usb_device.h>
#include <zephyr/usb/usbd.h>
#include <zephyr/drivers/uart.h>

#include <stdio.h>
#include <stdlib.h>

#include <nrfx_spim.h>
#define SPIM_INST_IDX 1

typedef struct
{
    int Param_A;
    int Param_B;
    int Param_C;
    int Param_D;
    int Param_E;
    int Param_Len;
} Param_Struct;

Param_Struct Param_Data_ON = {.Param_A = 0, .Param_B = 0, .Param_Len = 2};
Param_Struct Param_Data_Config_Mode = {.Param_A = 1, .Param_B = 0, .Param_C = 0, .Param_Len = 3};
Param_Struct Param_Data_Config_Channel = {.Param_A = 1, .Param_B = 1, .Param_C = 0, .Param_Len = 3};
Param_Struct Param_Data_Config_UpdateFre = {.Param_A = 1, .Param_B = 2, .Param_C = 0, .Param_Len = 3};
Param_Struct Param_Data_Config_TimeOut = {.Param_A = 1, .Param_B = 3, .Param_C = 0, .Param_Len = 3};

Param_Struct Param_Data;

#define BUFFER_SIZE_MULTI_CHANNEL (2 + 238 + 2)
#define BUFFER_SIZE_SINGLE_CHANNEL (2 + 150 + 2)

#define MOSI_PIN_MASTER 4
#define MISO_PIN_MASTER 5
#define SCK_PIN_MASTER 28
#define SS_PIN_MASTER 26

const struct device *ipc0_instance;
struct ipc_ept ep;

K_SEM_DEFINE(IPC_SEM, 0, 1);
K_SEM_DEFINE(TIMER_SEM, 0, 1);

uint8_t m_tx_buffer_master[4];
nrfx_spim_xfer_desc_t spim_xfer_desc_tx = NRFX_SPIM_XFER_TX(m_tx_buffer_master, 4);
nrfx_spim_t spim_inst = NRFX_SPIM_INSTANCE(SPIM_INST_IDX);

uint8_t IPC_Receive_Buffer[50];

LOG_MODULE_REGISTER(host, 4);

K_SEM_DEFINE(bound_sem, 0, 1);

#define _TAKE_SEM_()                               \
    do                                             \
    {                                              \
        if (k_sem_take(&IPC_SEM, _GET_TIMEOUT_())) \
            goto _RECOVERY_MODE_;                  \
    } while (0ULL)

#define _SPIM_WRITE_(ADDR, DATA)                           \
    do                                                     \
    {                                                      \
        m_tx_buffer_master[0] = ADDR >> 8;                 \
        m_tx_buffer_master[1] = ADDR & 0x00FF;             \
        m_tx_buffer_master[2] = DATA >> 8;                 \
        m_tx_buffer_master[3] = DATA & 0x00FF;             \
        nrfx_spim_xfer(&spim_inst, &spim_xfer_desc_tx, 0); \
    } while (0ULL)

k_timeout_t _GET_TIMEOUT_(void)
{
    switch (Param_Data_Config_TimeOut.Param_C)
    {
    case 0:
        return K_MSEC(10);
    case 1:
        return K_MSEC(20);
    case 2:
        return K_MSEC(40);
    case 3:
        return K_MSEC(100);
    case 4:
        return K_FOREVER;
    }
}

k_timeout_t _GET_UPDATE_FREQUENCY_(void)
{
    switch (Param_Data_Config_UpdateFre.Param_C)
    {
    case 0:
        return K_SECONDS(5);
    case 1:
        return K_SECONDS(3);
    case 2:
        return K_SECONDS(1);
    case 3:
        return K_SECONDS(30);
    case 4:
        return K_MINUTES(1);
    case 5:
        return K_MINUTES(5);
    }
}

void Timer_Handler(struct k_timer *timer_id)
{
    ARG_UNUSED(timer_id);
    // LOG_INF("Timer Handler");
    k_sem_give(&TIMER_SEM);
}

K_TIMER_DEFINE(Config_Timer, Timer_Handler, NULL);

// __INLINE void SPIM_Write_Reg(uint16_t addr, uint16_t data)
// {
//     m_tx_buffer_master[0] = addr >> 8;
//     m_tx_buffer_master[1] = addr & 0x00FF;
//     m_tx_buffer_master[2] = data >> 8;
//     m_tx_buffer_master[3] = data & 0x00FF;

//     nrfx_spim_xfer(&spim_inst, &spim_xfer_desc_tx, 0);
// }

static void spim_handler(nrfx_spim_evt_t const *p_event, void *p_context)
{
    if (p_event->type == NRFX_SPIM_EVENT_DONE)
    {
        char *p_msg = p_context;

        size_t tx_length = p_event->xfer_desc.tx_length;
        size_t rx_length = p_event->xfer_desc.rx_length;

        // NRFX_LOG_INFO("SPIM finished. >%s<", p_msg);

        if (tx_length > 0)
        {
        }

        if (rx_length > 0)
        {
        }
    }
}

static void ep_bound(void *priv)
{
    k_sem_give(&bound_sem);
    LOG_INF("Ep bounded");
}

void Power_Change(bool Power)
{
    // static bool Power_Status = false;
    // if (Power_Status == Power)
    //     return;
    // Power_Status = Power;
    _SPIM_WRITE_((uint16_t)0x0000, Power ? (uint16_t)0x0001 : (uint16_t)0x0000);
    // LOG_INF("Power change: %s", Power ? "ON" : "OFF");
}

void Mode_Change(bool Single_Channel)
{
    static bool Single_Channel_Status = false;
    if (Single_Channel_Status == Single_Channel)
        return;
    Single_Channel_Status = Single_Channel;
    _SPIM_WRITE_((uint16_t)0x000E, Single_Channel ? (uint16_t)0x0001 : (uint16_t)0x0000);
    LOG_INF("Mode change: %s", Single_Channel ? "Single-Channel" : "Multi-Channel");
}

void Set_Single_Channel(int Channel_Bit)
{
    static int Channel_Bit_Status = 0;
    if (Channel_Bit_Status == Channel_Bit)
        return;
    Channel_Bit_Status = Channel_Bit;
    _SPIM_WRITE_((uint16_t)0x000F, (uint16_t)Channel_Bit);
    LOG_INF("Channel change: %d", Channel_Bit);
}

void Change_Update_Frequency()
{
    static int Update_Frequency_Buffer = 0;

    if (Update_Frequency_Buffer == Param_Data_Config_UpdateFre.Param_C)
        return;
    k_timer_stop(&Config_Timer);
    Update_Frequency_Buffer = Param_Data_Config_UpdateFre.Param_C;
    k_timer_start(&Config_Timer, _GET_UPDATE_FREQUENCY_(), _GET_UPDATE_FREQUENCY_());
    // LOG_INF("Update frequency changed.");
}

void Commit_Change(void)
{
    LOG_INF("Commit Change");
    Mode_Change(Param_Data_Config_Mode.Param_C);
    Set_Single_Channel(Param_Data_Config_Channel.Param_C);
    Change_Update_Frequency();

    _SPIM_WRITE_((uint16_t)0x0000, Param_Data_ON.Param_B ? (uint16_t)0x0001 : (uint16_t)0x0000);
    // Power_Change(Param_Data_ON.Param_B);
}

bool Data_Parsing(void)
{
    if (IPC_Receive_Buffer[0] == 'E')
    {
        return true;
    }

    if (IPC_Receive_Buffer[0] == '9')
    {
        LOG_INF("DEBUG: %s", IPC_Receive_Buffer);
        return false;
    }

    Param_Data.Param_Len = 0;
    char *context;
    char *temp = strtok_r(IPC_Receive_Buffer, "-", &context);
    while (temp)
    {
        switch ((Param_Data.Param_Len)++)
        {
        case 0:
            Param_Data.Param_A = atoi(temp);
            break;
        case 1:
            Param_Data.Param_B = atoi(temp);
            break;
        case 2:
            Param_Data.Param_C = atoi(temp);
            break;
        case 3:
            Param_Data.Param_D = atoi(temp);
            break;
        case 4:
            Param_Data.Param_E = atoi(temp);
            break;
        }
        temp = strtok_r(NULL, "-", &context);
    }
    switch (Param_Data.Param_A)
    {
    case 0:
        if (Param_Data.Param_B == 0)
            Param_Data_ON.Param_B = 0;
        if (Param_Data.Param_B == 1)
            Param_Data_ON.Param_B = 1;
        break;
    case 1:
        if (Param_Data.Param_B == 0)
        {
            if (Param_Data.Param_C == 0)
                Param_Data_Config_Mode.Param_C = 0;
            if (Param_Data.Param_C == 1)
                Param_Data_Config_Mode.Param_C = 1;
        }
        else if (Param_Data.Param_B == 1)
        {
            Param_Data_Config_Channel.Param_C = Param_Data.Param_C;
        }
        else if (Param_Data.Param_B == 2)
        {
            Param_Data_Config_UpdateFre.Param_C = Param_Data.Param_C;
        }
        else if (Param_Data.Param_B == 3)
        {
            Param_Data_Config_TimeOut.Param_C = Param_Data.Param_C;
        }
        break;
    }
    return false;
}

static void ep_recv(const void *data, size_t len, void *priv)
{

    LOG_INF("EP RECV: %s", data);
}

static struct ipc_ept_cfg ep_cfg = {
    .cb = {
        .bound = ep_bound,
        .received = ep_recv,
    },
};

void init()
{
    int ret;

    LOG_INF("TX_NRF_CPUCORE");

    ipc0_instance = DEVICE_DT_GET(DT_NODELABEL(ipc0));

    ret = ipc_service_open_instance(ipc0_instance);
    if ((ret < 0) && (ret != -EALREADY))
    {
        LOG_ERR("ipc_service_open_instance() failure");
        return ret;
    }

    ret = ipc_service_register_endpoint(ipc0_instance, &ep, &ep_cfg);
    if (ret < 0)
    {
        LOG_ERR("ipc_service_register_endpoint() failure");
        return ret;
    }
}

int main(void)
{
    nrfx_err_t status;
    (void)status;

    init();
    return 0;
}

static int network_gpio_allow(void)
{

    //SPIS
    NRF_P0_S->PIN_CNF[27] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU << GPIO_PIN_CNF_MCUSEL_Pos);
    NRF_P0_S->PIN_CNF[29] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU << GPIO_PIN_CNF_MCUSEL_Pos);
    NRF_P0_S->PIN_CNF[30] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU << GPIO_PIN_CNF_MCUSEL_Pos);
    NRF_P0_S->PIN_CNF[31] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU << GPIO_PIN_CNF_MCUSEL_Pos);

    //SPIM
    NRF_P0_S->PIN_CNF[4] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU << GPIO_PIN_CNF_MCUSEL_Pos);
    NRF_P0_S->PIN_CNF[5] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU << GPIO_PIN_CNF_MCUSEL_Pos);
    NRF_P0_S->PIN_CNF[26] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU << GPIO_PIN_CNF_MCUSEL_Pos);
    // NRF_P0_S->PIN_CNF[24] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU << GPIO_PIN_CNF_MCUSEL_Pos);

    //GPIO
    NRF_P0_S->PIN_CNF[3] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU << GPIO_PIN_CNF_MCUSEL_Pos);
    NRF_P0_S->PIN_CNF[28] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU << GPIO_PIN_CNF_MCUSEL_Pos);
    
    NRF_P0_S->PIN_CNF[13] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU << GPIO_PIN_CNF_MCUSEL_Pos);
    NRF_P0_S->PIN_CNF[14] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU << GPIO_PIN_CNF_MCUSEL_Pos);
    NRF_P0_S->PIN_CNF[16] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU << GPIO_PIN_CNF_MCUSEL_Pos);
    NRF_P0_S->PIN_CNF[17] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU << GPIO_PIN_CNF_MCUSEL_Pos);
    NRF_P0_S->PIN_CNF[19] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU << GPIO_PIN_CNF_MCUSEL_Pos);
    NRF_P0_S->PIN_CNF[20] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU << GPIO_PIN_CNF_MCUSEL_Pos);


    	// uint32_t start_pin = (IS_ENABLED(CONFIG_SOC_ENABLE_LFXO) ? 2 : 0);

	/* Allow the network core to use all GPIOs. */
	// for (uint32_t i = start_pin; i < P0_PIN_NUM; i++) {
	// 	NRF_P0_S->PIN_CNF[i] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU <<
	// 				GPIO_PIN_CNF_MCUSEL_Pos);
	// }

	// for (uint32_t i = 0; i < P1_PIN_NUM; i++) {
	// 	NRF_P1_S->PIN_CNF[i] = (GPIO_PIN_CNF_MCUSEL_NetworkMCU <<
	// 				GPIO_PIN_CNF_MCUSEL_Pos);
	// }



    return 0;
}

SYS_INIT(network_gpio_allow, PRE_KERNEL_1, CONFIG_KERNEL_INIT_PRIORITY_OBJECTS);