#include <zephyr/drivers/clock_control.h>
#include <zephyr/drivers/clock_control/nrf_clock_control.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/irq.h>
#include <zephyr/logging/log.h>
#include <nrf.h>
#include <esb.h>
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/kernel.h>
#include <zephyr/types.h>
#include <zephyr/ipc/ipc_service.h>

#include <zephyr/drivers/gpio.h>

#include <string.h>
#include <stdlib.h>

#pragma region

#if !DT_NODE_EXISTS(DT_NODELABEL(software_spim))
#error "Overlay for software_spim node not properly defined."
#endif

static const struct gpio_dt_spec software_spim_mosi = GPIO_DT_SPEC_GET_BY_IDX_OR(DT_NODELABEL(software_spim), gpios, 0, {0});
static const struct gpio_dt_spec software_spim_miso = GPIO_DT_SPEC_GET_BY_IDX_OR(DT_NODELABEL(software_spim), gpios, 1, {0});
static const struct gpio_dt_spec software_spim_ss = GPIO_DT_SPEC_GET_BY_IDX_OR(DT_NODELABEL(software_spim), gpios, 2, {0});
static const struct gpio_dt_spec software_spim_sck = GPIO_DT_SPEC_GET_BY_IDX_OR(DT_NODELABEL(software_spim), gpios, 3, {0});

// static const struct gpio_dt_spec software_test = GPIO_DT_SPEC_GET_BY_IDX_OR(DT_NODELABEL(software_spim), gpios, 4, {0});

#pragma endregion

//5 13 14 16 17 19 20

static const struct gpio_dt_spec test_gpios[] = {
    GPIO_DT_SPEC_GET_BY_IDX_OR(DT_NODELABEL(test_gpio), gpios, 0, {0}),
    GPIO_DT_SPEC_GET_BY_IDX_OR(DT_NODELABEL(test_gpio), gpios, 1, {0}),
    GPIO_DT_SPEC_GET_BY_IDX_OR(DT_NODELABEL(test_gpio), gpios, 2, {0}),
    GPIO_DT_SPEC_GET_BY_IDX_OR(DT_NODELABEL(test_gpio), gpios, 3, {0}),
    GPIO_DT_SPEC_GET_BY_IDX_OR(DT_NODELABEL(test_gpio), gpios, 4, {0}),
    GPIO_DT_SPEC_GET_BY_IDX_OR(DT_NODELABEL(test_gpio), gpios, 5, {0}),
    GPIO_DT_SPEC_GET_BY_IDX_OR(DT_NODELABEL(test_gpio), gpios, 6, {0}),
};

#define BUFF_SIZE (250)



typedef struct spi_fifos
{
    sys_snode_t snode;
    uint8_t data[BUFF_SIZE];
    uint16_t data_size;
} spi_fifos_t;

spi_fifos_t spi_fifos;

volatile bool spi_start = false;

LOG_MODULE_REGISTER(esb_ptx, 4);
K_SEM_DEFINE(bound_sem, 0, 1);

K_SEM_DEFINE(SPI_SEM, 0, 1);
K_SEM_DEFINE(TIMER_SEM, 0, 1);
K_SEM_DEFINE(ESB_SEM, 0, 1);

K_MUTEX_DEFINE(ESB_MUTEX);

#define THREAD_STACK_SIZE 2048
#define THREAD_PRIORITY 5
#define THREAD_HIGH_PRIORITY 4

#include <nrfx_spis.h>
#define SPIS_INST_IDX 0

#define MOSI_PIN_SLAVE 31
#define MISO_PIN_SLAVE 30
#define SCK_PIN_SLAVE 29
#define CSN_PIN_SLAVE 27

#define UPDATE_PERIOD 30 // One data transfer after every nine data transfers carries an ACK message

static uint8_t m_tx_buffer_master[4];

nrfx_spis_t spis_inst = NRFX_SPIS_INSTANCE(SPIS_INST_IDX);

struct ipc_ept ep;
struct esb_config config = ESB_DEFAULT_CONFIG;
struct esb_payload tx_payload = {.pipe = 0, .length = BUFF_SIZE, .noack = false};
struct esb_payload rx_payload = ESB_CREATE_PAYLOAD(0, {0x00, 0x00, 0x00, 0x00, 0x00});

#define _TAKE_SEM_()                               \
    do                                             \
    {                                              \
        if (k_sem_take(&ESB_SEM, _GET_TIMEOUT_())) \
            goto _RECOVERY_MODE_;                  \
    } while (0ULL)

#define _SPIM_MOSI_(VALUE) gpio_pin_set_dt(&software_spim_mosi, !(VALUE))
#define _SPIM_CLK_(VALUE) gpio_pin_set_dt(&software_spim_sck, !(VALUE))
#define _SPIM_SS_(VALUE) gpio_pin_set_dt(&software_spim_ss, !(VALUE))
#define _SPIM_MISO_() gpio_pin_get_dt(&software_spim_miso)
#define _RHD_CONTROL_(VALUE) gpio_pin_set_dt(&(test_gpios[0]), !(VALUE))
// #define _TEST_(VALUE) gpio_pin_set_dt(&software_test, !(VALUE))
// #define _TEST_TOGGLE_() gpio_pin_toggle_dt(&software_test)
// #define _SPIM_DELAY_() k_sleep(K_MSEC(1))
#define _SPIM_DELAY_()               \
    do                               \
    {                                \
        uint32_t __i__ = 0x00FFFFFF; \
        while (__i__--)              \
            ;                        \
    } while (0ULL)
// #define _TEST_PIN_(VALUE) gpio_pin_set_dt(&software_test_pin, !(VALUE))
// #define _TEST_PIN_TOGGLE_() gpio_pin_toggle_dt(&software_test_pin)

#define DEBUG_LOG(INFO, ...)                              \
    {                                                     \
        char _BUFFER_[128];                               \
        int _LEN_ = sprintf(_BUFFER_, INFO, __VA_ARGS__); \
        ipc_service_send(&ep, _BUFFER_, _LEN_);           \
    }

K_FIFO_DEFINE(SPI_DATA_FIFO);

uint8_t Buffer_Send[50];
uint16_t Buffer_Get[5];
uint8_t Buffer_FIFO[BUFF_SIZE];

#define DUMMY_COMMAND 63
#define I_COMMAND 40
#define N1_COMMAND 41
#define T_COMMAND 42
#define A_COMMAND 43
#define N2_COMMAND 44

#define _READ_COMMAND_(COMMAND) (0xC000 | (COMMAND << 8))
#define _WRITE_(COMMAND) (0x8000 | COMMAND)
#define _WRITE_COMMAND_(REG, COMMAND) (_WRITE_(REG << 8 | COMMAND))
#define _CLEAR_COMMAND_ (0x6A00)
#define _CALIBRATE_COMMAND_ (0x5500)
#define _CONVERT_COMMAND_(CHANNEL, DSB) (CHANNEL << 8 & DSB)

uint8_t SPI_ReadByte(uint8_t data) __DEPRECATED("Using this function directly will cause the wrong command to be sent.");

void SPI_WriteByte(uint8_t data)
{
    uint8_t i = 0;
    uint8_t temp = 0;
    for (i = 0; i < 8; i++)
    {
        temp = ((data & 0x80) == 0x80) ? 1 : 0;
        data = data << 1;
        _SPIM_CLK_(0);
        _SPIM_MOSI_(temp);
        _SPIM_DELAY_();
        _SPIM_CLK_(1);
        _SPIM_DELAY_();
    }
    _SPIM_CLK_(0);
    _SPIM_MOSI_(0);
}

uint8_t SPI_ReadByte(uint8_t data)
{
    uint8_t i = 0;
    uint8_t read_data = 0xFF;
    for (i = 0; i < 8; i++)
    {
        read_data = read_data << 1;
        SPI_CLK(0);
        SPI_Delay();
        SPI_CLK(1);
        SPI_Delay();
        if (SPI_MISO() == 1)
        {
            read_data = read_data + 1;
        }
    }
    SPI_CLK(0);
    return read_data;
}

uint8_t SPI_WriteReadByte(uint8_t data)
{
    uint8_t i = 0;
    uint8_t temp = 0;
    uint8_t read_data = 0xFF;
    for (i = 0; i < 8; i++)
    {
        temp = ((data & 0x80) == 0x80) ? 1 : 0;
        data = data << 1;
        read_data = read_data << 1;
        _SPIM_CLK_(0);
        _SPIM_MOSI_(temp);
        _SPIM_DELAY_();
        _SPIM_CLK_(1);
        _SPIM_DELAY_();
        if (_SPI_MISO_() == 1)
        {
            read_data = read_data + 1;
        }
    }
    _SPIM_CLK_(0);
    return read_data;
}

void SPI_Write(uint16_t addr, uint16_t data)
{
    uint8_t buffer[4] = {0};

    buffer[0] = addr >> 8;
    buffer[1] = addr & 0x00FF;
    buffer[2] = data >> 8;
    buffer[3] = data & 0x00FF;

    _SPIM_SS_(0);

    for (uint8_t i = 0; i < 4; i++)
        SPI_WriteByte(buffer[i]);

    _SPIM_SS_(1);
}

uint16_t SPI_WriteReadWord(uint16_t data)
{
    uint16_t i = 0;
    uint16_t temp = 0;
    uint16_t read_data = 0xFFFF;

    _SPIM_SS_(0);

    for (i = 0; i < 16; i++)
    {
        temp = ((data & 0x8000) == 0x8000) ? 1 : 0;
        data = data << 1;
        read_data = read_data << 1;
        _SPIM_CLK_(0);
        _SPIM_MOSI_(temp);
        _SPIM_DELAY_();
        _SPIM_CLK_(1);
        _SPIM_DELAY_();
        if (_SPIM_MISO_() == 0)
        {
            read_data = read_data + 1;
        }
    }

    _SPIM_CLK_(0);
    _SPIM_MOSI_(0);
    _SPIM_SS_(1);

    return read_data;
}

uint16_t RHD_Read(uint16_t data)
{
    uint16_t datas = 0x0000;
    SPI_WriteReadWord(data);
    SPI_WriteReadWord(DUMMY_COMMAND);
    datas = SPI_WriteReadWord(DUMMY_COMMAND);
    return datas;
}

void Power_Change(bool Power)
{
    static bool Power_Status = false;
    if (Power_Status == Power)
        return;
    Power_Status = Power;
    SPI_Write((uint16_t)0x0000, Power ? (uint16_t)0x0001 : (uint16_t)0x0000);
}

void Mode_Change(bool Single_Channel)
{
    static bool Single_Channel_Status = false;
    if (Single_Channel_Status == Single_Channel)
        return;
    Single_Channel_Status = Single_Channel;
    SPI_Write((uint16_t)0x0000, (uint16_t)0x0000);
    _SPIM_DELAY_();

    for(int i = 0; i < 2; i++)
    {
        SPI_Write((uint16_t)0x000E, Single_Channel ? (uint16_t)0x0001 : (uint16_t)0x0000);
        _SPIM_DELAY_();   
    }
    while(!k_fifo_is_empty(&SPI_DATA_FIFO)) k_fifo_get(&SPI_DATA_FIFO, K_NO_WAIT);
    SPI_Write((uint16_t)0x0000, (uint16_t)0x0001);
}

void Set_Single_Channel(unsigned char Channel[])
{
    static uint32_t Channel_Bit;
    uint32_t _Channel_Bit = 0x00000000 | (1 << Channel[0]) | (1 << Channel[1]) | (1 << Channel[2]) | (1 << Channel[3]);
    if (Channel_Bit == _Channel_Bit) return;
    Channel_Bit = _Channel_Bit;
    SPI_Write((uint16_t)0x0000, (uint16_t)0x0000);
    _SPIM_DELAY_();

    for(int i = 0; i < 2; i++)
    {
        SPI_Write((uint16_t)0x000F, (uint16_t)Channel_Bit & 0x0000FFFF);
        _SPIM_DELAY_();
        SPI_Write((uint16_t)0x0020, ((uint16_t)Channel_Bit & 0xFFFF0000) >> 16);
        _SPIM_DELAY_();
    }
    while(!k_fifo_is_empty(&SPI_DATA_FIFO)) k_fifo_get(&SPI_DATA_FIFO, K_NO_WAIT);
    SPI_Write((uint16_t)0x0000, (uint16_t)0x0001);
}

bool Data_Parsing(void)
{
    unsigned char _Channel_Bit[] = {rx_payload.data[1], rx_payload.data[2], rx_payload.data[3], rx_payload.data[4]};
    Set_Single_Channel(_Channel_Bit);
    Mode_Change(rx_payload.data[0]);
    // gpio_pin_toggle_dt(&(test_gpios[1]));
    // gpio_pin_set_dt(&(test_gpios[1]), rx_payload.data[0]);
}

static void spis_handler(nrfx_spis_evt_t const *p_event, void *p_context)
{
    if (p_event->evt_type == NRFX_SPIS_XFER_DONE)
    {
        nrfx_err_t status;
        (void)status;

        nrfx_spis_t *p_spis_inst = p_context;

        size_t rx_length = p_event->rx_amount;

        if (rx_length > 0)
        {
  
            spi_fifos.data_size = rx_length;
            // tx_payload.length = rx_length;
            // esb_write_payload(&tx_payload);
            memcpy(spi_fifos.data, Buffer_FIFO, rx_length);

            k_fifo_put(&SPI_DATA_FIFO, &spi_fifos);
            gpio_pin_toggle_dt(&(test_gpios[2]));
            // nrfx_spis_buffers_set(p_spis_inst, NULL, 0, tx_payload.data, BUFF_SIZE);
            nrfx_spis_buffers_set(p_spis_inst, NULL, 0, Buffer_FIFO, BUFF_SIZE);
            
        }
    }
}

uint8_t status = 0;
uint32_t _count_ = UPDATE_PERIOD;

void event_handler(struct esb_evt const *event)
{
    switch (event->evt_id)
    {
    case ESB_EVENT_TX_SUCCESS:
        break;
    case ESB_EVENT_TX_FAILED:

        break;
    case ESB_EVENT_RX_RECEIVED:
        esb_read_rx_payload(&rx_payload);      
        break;
    }
}

int clocks_start(void)
{
    int err;
    int res;
    struct onoff_manager *clk_mgr;
    struct onoff_client clk_cli;

    clk_mgr = z_nrf_clock_control_get_onoff(CLOCK_CONTROL_NRF_SUBSYS_HF);
    if (!clk_mgr)
    {
        return -ENXIO;
    }

    sys_notify_init_spinwait(&clk_cli.notify);

    err = onoff_request(clk_mgr, &clk_cli);
    if (err < 0)
    {
        return err;
    }

    do
    {
        err = sys_notify_fetch_result(&clk_cli.notify, &res);
        if (!err && res)
        {
            return res;
        }
    } while (err);

    return 0;
}

int esb_initialize(void)
{
    int err;

    uint8_t base_addr_0[4] = {0xA0, 0xA0, 0xA0, 0xA0};
    uint8_t base_addr_1[4] = {0x00, 0x00, 0x00, 0x00};
    uint8_t addr_prefix[ ] = {0xA0};

    config.protocol = ESB_PROTOCOL_ESB_DPL;
    config.retransmit_delay = 150;
    config.bitrate = ESB_BITRATE_2MBPS;
    config.event_handler = event_handler;
    config.mode = ESB_MODE_PTX;
    config.payload_length = 252;
    config.tx_mode = ESB_TXMODE_AUTO;
    // config.tx_mode = ESB_TXMODE_MANUAL_START;
    config.use_fast_ramp_up = true;
    config.selective_auto_ack = true;
    config.tx_output_power = ESB_TX_POWER_0DBM;
    config.retransmit_count = 0;
    config.crc = ESB_CRC_OFF;

    err = esb_init(&config);
    if (err)
    {
        return err;
    }

    err = esb_set_base_address_0(base_addr_0);
    if (err)
    {
        return err;
    }

    err = esb_set_base_address_1(base_addr_1);
    if (err)
    {
        return err;
    }

    err = esb_set_prefixes(addr_prefix, ARRAY_SIZE(addr_prefix));
    if (err)
    {
        return err;
    }

    return 0;
}

static void ep_bound(void *priv)
{
    k_sem_give(&bound_sem);
}

static void ep_recv(const void *data, size_t len, void *priv)
{
}

static struct ipc_ept_cfg ep_cfg = {
    .cb = {
        .bound = ep_bound,
        .received = ep_recv,
    },
};

void esb_ipc_init(void)
{
    const struct device *ipc0_instance;
    int err;

    ipc0_instance = DEVICE_DT_GET(DT_NODELABEL(ipc0));

    err = clocks_start();
    if (err)
    {
        return;
    }

    err = ipc_service_open_instance(ipc0_instance);
    if ((err < 0) && (err != -EALREADY))
    {
        return;
    }

    err = ipc_service_register_endpoint(ipc0_instance, &ep, &ep_cfg);
    if (err != 0)
    {
        return;
    }

    err = esb_initialize();
    if (err)
    {
        return;
    }

    k_sem_take(&bound_sem, K_FOREVER);
}

void spis_init(void)
{
    nrfx_err_t status;
    (void)status;
    nrfx_spis_config_t spis_config = NRFX_SPIS_DEFAULT_CONFIG(SCK_PIN_SLAVE,
                                                              MOSI_PIN_SLAVE,
                                                              MISO_PIN_SLAVE,
                                                              CSN_PIN_SLAVE);
    spis_config.mode = 1;
    status = nrfx_spis_init(&spis_inst, &spis_config, spis_handler, &spis_inst);
    NRFX_ASSERT(status == NRFX_SUCCESS);

#if defined(__ZEPHYR__)

    IRQ_DIRECT_CONNECT(NRFX_IRQ_NUMBER_GET(NRF_SPIS_INST_GET(SPIS_INST_IDX)), IRQ_PRIO_LOWEST,
                       NRFX_SPIS_INST_HANDLER_GET(SPIS_INST_IDX), 0);

#endif

    status = nrfx_spis_buffers_set(&spis_inst, NULL, 0, Buffer_FIFO, BUFF_SIZE);
    // status = nrfx_spis_buffers_set(&spis_inst, NULL, 0, tx_payload.data, BUFF_SIZE);
    NRFX_ASSERT(status == NRFX_SUCCESS);
}

void spim_init(void)
{

    gpio_is_ready_dt(&software_spim_mosi);
    gpio_is_ready_dt(&software_spim_miso);
    gpio_is_ready_dt(&software_spim_sck);
    gpio_is_ready_dt(&software_spim_ss);

    gpio_pin_configure_dt(&software_spim_mosi, GPIO_OUTPUT_INACTIVE);
    gpio_pin_configure_dt(&software_spim_miso, GPIO_INPUT);
    gpio_pin_configure_dt(&software_spim_sck, GPIO_OUTPUT_INACTIVE);
    gpio_pin_configure_dt(&software_spim_ss, GPIO_OUTPUT_INACTIVE);

    k_sleep(K_MSEC(1));

    _SPIM_CLK_(0);
    _SPIM_MOSI_(0);
    _SPIM_SS_(1);
}

void gpio_init(void)
{
    int err;

    for (int i = 0; i < 7; i++)
    {
        gpio_is_ready_dt(&test_gpios[i]);
        gpio_pin_configure_dt(&test_gpios[i], GPIO_OUTPUT_INACTIVE);
        gpio_pin_set_dt(&test_gpios[i], !(0));
    }
}

void RHD_Init(void)
{
    SPI_WriteReadWord(_READ_COMMAND_(DUMMY_COMMAND));
    SPI_WriteReadWord(_READ_COMMAND_(DUMMY_COMMAND));

    SPI_WriteReadWord(_WRITE_COMMAND_(0, 0xDE));

    SPI_WriteReadWord(_WRITE_COMMAND_(1, 0x42));
    SPI_WriteReadWord(_WRITE_COMMAND_(2, 0x04));

    SPI_WriteReadWord(_WRITE_COMMAND_(3, 0x00));

    SPI_WriteReadWord(_WRITE_COMMAND_(4, 0x80));

    SPI_WriteReadWord(_WRITE_COMMAND_(5, 0x40));
    SPI_WriteReadWord(_WRITE_COMMAND_(6, 0x80));
    SPI_WriteReadWord(_WRITE_COMMAND_(7, 0x00));

    SPI_WriteReadWord(_WRITE_COMMAND_(8, 0x16));
    SPI_WriteReadWord(_WRITE_COMMAND_(9, 0x80));
    SPI_WriteReadWord(_WRITE_COMMAND_(10, 0x17));
    SPI_WriteReadWord(_WRITE_COMMAND_(11, 0x80));

    SPI_WriteReadWord(_WRITE_COMMAND_(12, 0x2C));
    SPI_WriteReadWord(_WRITE_COMMAND_(13, 0x86));

    SPI_WriteReadWord(_WRITE_COMMAND_(14, 0xFF));
    SPI_WriteReadWord(_WRITE_COMMAND_(15, 0xFF));
    SPI_WriteReadWord(_WRITE_COMMAND_(16, 0xFF));
    SPI_WriteReadWord(_WRITE_COMMAND_(17, 0xFF));

    SPI_WriteReadWord(_READ_COMMAND_(DUMMY_COMMAND));
    SPI_WriteReadWord(_READ_COMMAND_(DUMMY_COMMAND));
    SPI_WriteReadWord(_READ_COMMAND_(DUMMY_COMMAND));
    SPI_WriteReadWord(_READ_COMMAND_(DUMMY_COMMAND));
    SPI_WriteReadWord(_READ_COMMAND_(DUMMY_COMMAND));
    SPI_WriteReadWord(_READ_COMMAND_(DUMMY_COMMAND));
    SPI_WriteReadWord(_READ_COMMAND_(DUMMY_COMMAND));
    SPI_WriteReadWord(_READ_COMMAND_(DUMMY_COMMAND));
    SPI_WriteReadWord(_READ_COMMAND_(DUMMY_COMMAND));
}

void FPGA_Init(void)
{
    _RHD_CONTROL_(false);
    k_sleep(K_MSEC(10));

    SPI_Write((uint16_t)0x0000, (uint16_t)0x0000);
    k_sleep(K_MSEC(10));

    _RHD_CONTROL_(true);
    k_sleep(K_MSEC(10));

    RHD_Init();
    k_sleep(K_MSEC(10));

    _RHD_CONTROL_(false);
    k_sleep(K_MSEC(10));

    // SPI_Write((uint16_t)0x000F, (uint16_t)0x000F);
    // k_sleep(K_MSEC(10));

    SPI_Write((uint16_t)0x000E, (uint16_t)0x0000);
    k_sleep(K_MSEC(10));

    SPI_Write((uint16_t)0x0000, (uint16_t)0x0001);
    k_sleep(K_MSEC(10));
}

int main(void)
{
    esb_ipc_init();
    spis_init();
    spim_init();
    gpio_init();

    FPGA_Init();

    return 0;
}

void SPI_Thread(void *p1, void *p2, void *p3)
{
    spi_fifos_t *rx_data;
    while (1)
    {
        // if(esb_is_idle()) Data_Parsing();
        // esb_start_tx();
        // gpio_pin_toggle_dt(&(test_gpios[1]));
        if (likely(!k_fifo_is_empty(&SPI_DATA_FIFO)))
        {
            rx_data = k_fifo_get(&SPI_DATA_FIFO, K_FOREVER);
            if (rx_data)
            {
                memcpy(tx_payload.data, rx_data->data, rx_data->data_size);
                tx_payload.length = rx_data->data_size;

                esb_write_payload(&tx_payload);
                
                gpio_pin_toggle_dt(&(test_gpios[1]));
            }
        }
    }
}

K_THREAD_DEFINE(SPI_THREAD_ID, THREAD_STACK_SIZE, SPI_Thread, NULL, NULL, NULL, THREAD_HIGH_PRIORITY, 0, 0);

